var divide = function (y, x) { // divide y by x without using the division operator "/"
	var num,
		decompose = function (x: number) { // decompose Number x into { sign, exponent, fraction }
			var sign = 1,
				exponent = 0; // sign 1 or -1, base 2 exponent
			if (x < 0) {
				x = -x;
				sign = -1;
			}
			while (x >= 1) {
				x = 0.5 * x;
				exponent += 1;
			}
			while (x < 0.5) {
				x = 2 * x;
				exponent -= 1;
			}
			return {
				sign: sign,
				exponent: exponent,
				fraction: x
			}; // fraction	0.5 <= x < 1
		},

		compose = function (sign: number, exponent: number, fraction: number) {
			var x = fraction;
			while (exponent > 0) {
				x = 2 * x;
				exponent -= 1;
			}
			while (exponent < 0) {
				x = 0.5 * x;
				exponent += 1;
			}
			return sign * x;
		},
		
		reciprocal = function (x: number) {	// 0.5 <= x < 1
			var a = 2.666666666666666667,
				b = -6.0,
				c = 4.333333333333333333,
				y,
				i;
			y = (a * x + b) * x + c;		// first guess using quadratic approximation
			for (i = 0; i < 5; i += 1) {
				y = y * (2.0 - y * x);		// Newton iteration
			}
			return y;	// 1/x
		};

	y = Number(y);
	x = Number(x);
	if (isNaN(y) || isNaN(x)) {
		return NaN;
	}
	if (x === 0) {
		if (y === 0) {
			return NaN;
		}
		if (y >= 0) {
			return Infinity;
		}
		return -Infinity;
	}
	num = decompose(x);
	return y * compose(num.sign, -num.exponent, reciprocal(num.fraction));
};

var test = function (y, x) {
	print("Software: " + divide(y, x).toPrecision(21));
	print("Hardware: " + (y / x).toPrecision(21));
};

test(1, 7);
test(1, 10);
test(22, 7);
